# coding: utf-8

"""
    NetHSM
    All endpoints expect exactly the specified JSON. Additional properties will cause a Bad Request Error (400). All HTTP errors contain a JSON structure with an explanation of type string. All [base64](https://tools.ietf.org/html/rfc4648#section-4) encoded values are Big Endian.   # noqa: E501
    The version of the OpenAPI document: v1
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from __future__ import annotations
from nethsm.client.shared_imports.schema_imports import *  # pyright: ignore [reportWildcardImportFromLibrary]

CountryName: typing_extensions.TypeAlias = schemas.StrSchema
StateOrProvinceName: typing_extensions.TypeAlias = schemas.StrSchema
LocalityName: typing_extensions.TypeAlias = schemas.StrSchema
OrganizationName: typing_extensions.TypeAlias = schemas.StrSchema
OrganizationalUnitName: typing_extensions.TypeAlias = schemas.StrSchema
CommonName: typing_extensions.TypeAlias = schemas.StrSchema
EmailAddress: typing_extensions.TypeAlias = schemas.StrSchema
Items: typing_extensions.TypeAlias = schemas.StrSchema


class SubjectAltNamesTuple(
    typing.Tuple[
        str,
        ...
    ]
):

    def __new__(cls, arg: typing.Union[SubjectAltNamesTupleInput, SubjectAltNamesTuple], configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None):
        return SubjectAltNames.validate(arg, configuration=configuration)
SubjectAltNamesTupleInput = typing.Union[
    typing.List[
        str,
    ],
    typing.Tuple[
        str,
        ...
    ]
]


@dataclasses.dataclass(frozen=True)
class SubjectAltNames(
    schemas.Schema[schemas.immutabledict, SubjectAltNamesTuple]
):
    types: typing.FrozenSet[typing.Type] = frozenset({tuple})
    items: typing.Type[Items] = dataclasses.field(default_factory=lambda: Items) # type: ignore
    type_to_output_cls: typing.Mapping[
        typing.Type,
        typing.Type
    ] = dataclasses.field(
        default_factory=lambda: {
            tuple: SubjectAltNamesTuple
        }
    )

    @classmethod
    def validate(
        cls,
        arg: typing.Union[
            SubjectAltNamesTupleInput,
            SubjectAltNamesTuple,
        ],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> SubjectAltNamesTuple:
        return super().validate_base(
            arg,
            configuration=configuration,
        )
Properties = typing.TypedDict(
    'Properties',
    {
        "countryName": typing.Type[CountryName],
        "stateOrProvinceName": typing.Type[StateOrProvinceName],
        "localityName": typing.Type[LocalityName],
        "organizationName": typing.Type[OrganizationName],
        "organizationalUnitName": typing.Type[OrganizationalUnitName],
        "commonName": typing.Type[CommonName],
        "emailAddress": typing.Type[EmailAddress],
        "subjectAltNames": typing.Type[SubjectAltNames],
    }
)


class DistinguishedNameDict(schemas.immutabledict[str, schemas.OUTPUT_BASE_TYPES]):

    __required_keys__: typing.FrozenSet[str] = frozenset({
        "commonName",
    })
    __optional_keys__: typing.FrozenSet[str] = frozenset({
        "countryName",
        "stateOrProvinceName",
        "localityName",
        "organizationName",
        "organizationalUnitName",
        "emailAddress",
        "subjectAltNames",
    })
    
    def __new__(
        cls,
        *,
        commonName: str,
        countryName: typing.Union[
            str,
            schemas.Unset
        ] = schemas.unset,
        stateOrProvinceName: typing.Union[
            str,
            schemas.Unset
        ] = schemas.unset,
        localityName: typing.Union[
            str,
            schemas.Unset
        ] = schemas.unset,
        organizationName: typing.Union[
            str,
            schemas.Unset
        ] = schemas.unset,
        organizationalUnitName: typing.Union[
            str,
            schemas.Unset
        ] = schemas.unset,
        emailAddress: typing.Union[
            str,
            schemas.Unset
        ] = schemas.unset,
        subjectAltNames: typing.Union[
            SubjectAltNamesTupleInput,
            SubjectAltNamesTuple,
            schemas.Unset
        ] = schemas.unset,
        configuration_: typing.Optional[schema_configuration.SchemaConfiguration] = None,
        **kwargs: schemas.INPUT_TYPES_ALL,
    ):
        arg_: typing.Dict[str, typing.Any] = {
            "commonName": commonName,
        }
        for key_, val in (
            ("countryName", countryName),
            ("stateOrProvinceName", stateOrProvinceName),
            ("localityName", localityName),
            ("organizationName", organizationName),
            ("organizationalUnitName", organizationalUnitName),
            ("emailAddress", emailAddress),
            ("subjectAltNames", subjectAltNames),
        ):
            if isinstance(val, schemas.Unset):
                continue
            arg_[key_] = val
        arg_.update(kwargs)
        used_arg_ = typing.cast(DistinguishedNameDictInput, arg_)
        return DistinguishedName.validate(used_arg_, configuration=configuration_)
    
    @staticmethod
    def from_dict_(
        arg: typing.Union[
            DistinguishedNameDictInput,
            DistinguishedNameDict
        ],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> DistinguishedNameDict:
        return DistinguishedName.validate(arg, configuration=configuration)
    
    @property
    def commonName(self) -> str:
        return typing.cast(
            str,
            self.__getitem__("commonName")
        )
    
    @property
    def countryName(self) -> typing.Union[str, schemas.Unset]:
        val = self.get("countryName", schemas.unset)
        if isinstance(val, schemas.Unset):
            return val
        return typing.cast(
            str,
            val
        )
    
    @property
    def stateOrProvinceName(self) -> typing.Union[str, schemas.Unset]:
        val = self.get("stateOrProvinceName", schemas.unset)
        if isinstance(val, schemas.Unset):
            return val
        return typing.cast(
            str,
            val
        )
    
    @property
    def localityName(self) -> typing.Union[str, schemas.Unset]:
        val = self.get("localityName", schemas.unset)
        if isinstance(val, schemas.Unset):
            return val
        return typing.cast(
            str,
            val
        )
    
    @property
    def organizationName(self) -> typing.Union[str, schemas.Unset]:
        val = self.get("organizationName", schemas.unset)
        if isinstance(val, schemas.Unset):
            return val
        return typing.cast(
            str,
            val
        )
    
    @property
    def organizationalUnitName(self) -> typing.Union[str, schemas.Unset]:
        val = self.get("organizationalUnitName", schemas.unset)
        if isinstance(val, schemas.Unset):
            return val
        return typing.cast(
            str,
            val
        )
    
    @property
    def emailAddress(self) -> typing.Union[str, schemas.Unset]:
        val = self.get("emailAddress", schemas.unset)
        if isinstance(val, schemas.Unset):
            return val
        return typing.cast(
            str,
            val
        )
    
    @property
    def subjectAltNames(self) -> typing.Union[SubjectAltNamesTuple, schemas.Unset]:
        val = self.get("subjectAltNames", schemas.unset)
        if isinstance(val, schemas.Unset):
            return val
        return typing.cast(
            SubjectAltNamesTuple,
            val
        )
    
    def get_additional_property_(self, name: str) -> typing.Union[schemas.OUTPUT_BASE_TYPES, schemas.Unset]:
        schemas.raise_if_key_known(name, self.__required_keys__, self.__optional_keys__)
        return self.get(name, schemas.unset)
DistinguishedNameDictInput = typing.Mapping[str, schemas.INPUT_TYPES_ALL]


@dataclasses.dataclass(frozen=True)
class DistinguishedName(
    schemas.Schema[DistinguishedNameDict, tuple]
):
    """NOTE: This class is auto generated by OpenAPI JSON Schema Generator.
    Ref: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator

    Do not edit the class manually.

    Data used for creation of CSRs.
If subjectAltNames is omitted, it is set to the same value as
commonName. If it is set to an empty list, no SAN Extension is added.
All SANs are considered DNS names, unless they start with "IP:" to signal
IP names.

    """
    types: typing.FrozenSet[typing.Type] = frozenset({schemas.immutabledict})
    required: typing.FrozenSet[str] = frozenset({
        "commonName",
    })
    properties: Properties = dataclasses.field(default_factory=lambda: schemas.typed_dict_to_instance(Properties)) # type: ignore
    type_to_output_cls: typing.Mapping[
        typing.Type,
        typing.Type
    ] = dataclasses.field(
        default_factory=lambda: {
            schemas.immutabledict: DistinguishedNameDict
        }
    )

    @classmethod
    def validate(
        cls,
        arg: typing.Union[
            DistinguishedNameDictInput,
            DistinguishedNameDict,
        ],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> DistinguishedNameDict:
        return super().validate_base(
            arg,
            configuration=configuration,
        )




# Based on export_backup.py from the NetHSM source by Sven Anderson <sven@anderson.de>

import hashlib
import struct
from dataclasses import dataclass, field
from typing import Optional

from cryptography.exceptions import InvalidTag
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes


def _get_length(data: memoryview) -> tuple[int, memoryview]:
    if len(data) < 3:
        raise ValueError("Failed to read field length: unexpected EOF")
    high, low = struct.unpack(">B H", data[:3])
    return ((high << 16) + low, data[3:])


def _get_field(data: memoryview) -> tuple[bytes, memoryview]:
    n, data = _get_length(data)
    if len(data) < n:
        raise ValueError("Failed to extract field: unexpected EOF")
    return data[:n].tobytes(), data[n:]


def _decrypt(key: bytes, adata: bytes, data: bytes) -> bytes:
    iv_size = 12
    ciphertext = data[iv_size:-16]
    tag = data[-16:]
    nonce = data[:iv_size]

    cipher = Cipher(algorithms.AES(key), modes.GCM(nonce, tag), backend=default_backend())
    decryptor = cipher.decryptor()
    decryptor.authenticate_additional_data(adata)

    try:
        decrypted_data = decryptor.update(ciphertext) + decryptor.finalize()
        return decrypted_data
    except InvalidTag as e:
        raise ValueError("Authentication tag verification failed. The data may be tampered.") from e


@dataclass
class Backup:
    version: int
    domain_key: bytes
    data: dict[str, bytes] = field(default_factory=dict)
    backup_device_id: Optional[bytes] = None
    backup_config_store_key: Optional[bytes] = None


@dataclass
class EncryptedBackup:
    version: int
    salt: bytes
    encrypted_version: bytes
    encrypted_domain_key: bytes
    encrypted_data: list[bytes] = field(default_factory=list)
    encrypted_backup_device_id: Optional[bytes] = None
    encrypted_backup_config_store_key: Optional[bytes] = None

    @classmethod
    def parse(cls, data: bytes) -> "EncryptedBackup":
        mv = memoryview(data)
        header = b"_NETHSM_BACKUP_"
        header_len = len(header)
        if len(mv) < header_len + 1 or mv[:header_len] != header:
            raise ValueError("Data does not contain a NetHSM header")
        version = mv[header_len]
        mv = mv[header_len + 1 :]

        if version not in [0, 1]:
            raise ValueError(
                f"Version mismatch on export, provided backup version is {version}, this tool expects 0 or 1"
            )

        if version > 0:
            trailer = b"\x00\x00\x00_NETHSM_BACKUP_END_"
            trailer_len = len(trailer)
            if len(mv) < trailer_len or mv[-trailer_len:] != trailer:
                raise ValueError("Data is truncated (missing NetHSM trailer)")
            mv = mv[:-trailer_len]

        salt, mv = _get_field(mv)
        encrypted_version, mv = _get_field(mv)
        encrypted_domain_key, mv = _get_field(mv)

        encrypted_backup_device_id = None
        encrypted_backup_config_store_key = None
        if version > 0:
            encrypted_backup_device_id, mv = _get_field(mv)
            encrypted_backup_config_store_key, mv = _get_field(mv)

        backup = cls(
            version=version,
            salt=salt,
            encrypted_version=encrypted_version,
            encrypted_domain_key=encrypted_domain_key,
            encrypted_backup_device_id=encrypted_backup_device_id,
            encrypted_backup_config_store_key=encrypted_backup_config_store_key,
        )

        while mv:
            item, mv = _get_field(mv)
            backup.encrypted_data.append(item)

        return backup

    def _key(self, passphrase: str) -> bytes:
        return hashlib.scrypt(
            password=passphrase.encode(), salt=self.salt, n=16384, r=8, p=16, dklen=32
        )

    def decrypt(self, passphrase: str) -> Backup:
        key = self._key(passphrase)
        version_bytes = _decrypt(key, b"backup-version", self.encrypted_version)
        if len(version_bytes) != 1:
            raise ValueError(f"Overlong version: {version_bytes!r}")
        version = version_bytes[0]
        if version != self.version:
            raise ValueError(
                f"Internal and external version mismatch ({version} != {self.version})."
            )
        domain_key = _decrypt(key, b"domain-key", self.encrypted_domain_key)
        backup_device_id = None
        if self.encrypted_backup_device_id is not None:
            backup_device_id = _decrypt(key, b"backup-device-id", self.encrypted_backup_device_id)
        backup_config_store_key = None
        if self.encrypted_backup_config_store_key is not None:
            backup_config_store_key = _decrypt(
                key, b"backup-config-store-key", self.encrypted_backup_config_store_key
            )

        backup = Backup(
            version=version,
            domain_key=domain_key,
            backup_device_id=backup_device_id,
            backup_config_store_key=backup_config_store_key,
        )

        for item in self.encrypted_data:
            key_value_pair = _decrypt(key, b"backup", item)
            k, v = _get_field(memoryview(key_value_pair))
            backup.data[k.decode()] = v.tobytes()

        return backup



# coding: utf-8

"""
    NetHSM
    All endpoints expect exactly the specified JSON. Additional properties will cause a Bad Request Error (400). All HTTP errors contain a JSON structure with an explanation of type string. All [base64](https://tools.ietf.org/html/rfc4648#section-4) encoded values are Big Endian.   # noqa: E501
    The version of the OpenAPI document: v1
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from __future__ import annotations
from nethsm.client.shared_imports.schema_imports import *  # pyright: ignore [reportWildcardImportFromLibrary]

JoinerKit: typing_extensions.TypeAlias = schemas.StrSchema


class ClusterMemberAddResponseDict(schemas.immutabledict[str, schemas.OUTPUT_BASE_TYPES]):

    __required_keys__: typing.FrozenSet[str] = frozenset({
        "joinerKit",
        "members",
    })
    __optional_keys__: typing.FrozenSet[str] = frozenset({
    })
    
    def __new__(
        cls,
        *,
        joinerKit: str,
        members: typing.Union[
            MembersTupleInput,
            MembersTuple
        ],
        configuration_: typing.Optional[schema_configuration.SchemaConfiguration] = None,
        **kwargs: schemas.INPUT_TYPES_ALL,
    ):
        arg_: typing.Dict[str, typing.Any] = {
            "joinerKit": joinerKit,
            "members": members,
        }
        arg_.update(kwargs)
        used_arg_ = typing.cast(ClusterMemberAddResponseDictInput, arg_)
        return ClusterMemberAddResponse.validate(used_arg_, configuration=configuration_)
    
    @staticmethod
    def from_dict_(
        arg: typing.Union[
            ClusterMemberAddResponseDictInput,
            ClusterMemberAddResponseDict
        ],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> ClusterMemberAddResponseDict:
        return ClusterMemberAddResponse.validate(arg, configuration=configuration)
    
    @property
    def joinerKit(self) -> str:
        return typing.cast(
            str,
            self.__getitem__("joinerKit")
        )
    
    @property
    def members(self) -> MembersTuple:
        return typing.cast(
            MembersTuple,
            self.__getitem__("members")
        )
    
    def get_additional_property_(self, name: str) -> typing.Union[schemas.OUTPUT_BASE_TYPES, schemas.Unset]:
        schemas.raise_if_key_known(name, self.__required_keys__, self.__optional_keys__)
        return self.get(name, schemas.unset)
ClusterMemberAddResponseDictInput = typing.Mapping[str, schemas.INPUT_TYPES_ALL]


@dataclasses.dataclass(frozen=True)
class ClusterMemberAddResponse(
    schemas.Schema[ClusterMemberAddResponseDict, tuple]
):
    """NOTE: This class is auto generated by OpenAPI JSON Schema Generator.
    Ref: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator

    Do not edit the class manually.
    """
    types: typing.FrozenSet[typing.Type] = frozenset({schemas.immutabledict})
    required: typing.FrozenSet[str] = frozenset({
        "joinerKit",
        "members",
    })
    properties: Properties = dataclasses.field(default_factory=lambda: schemas.typed_dict_to_instance(Properties)) # type: ignore
    type_to_output_cls: typing.Mapping[
        typing.Type,
        typing.Type
    ] = dataclasses.field(
        default_factory=lambda: {
            schemas.immutabledict: ClusterMemberAddResponseDict
        }
    )

    @classmethod
    def validate(
        cls,
        arg: typing.Union[
            ClusterMemberAddResponseDictInput,
            ClusterMemberAddResponseDict,
        ],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> ClusterMemberAddResponseDict:
        return super().validate_base(
            arg,
            configuration=configuration,
        )


from nethsm.client.components.schema import cluster_initial_member


class MembersTuple(
    typing.Tuple[
        cluster_initial_member.ClusterInitialMemberDict,
        ...
    ]
):

    def __new__(cls, arg: typing.Union[MembersTupleInput, MembersTuple], configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None):
        return Members.validate(arg, configuration=configuration)
MembersTupleInput = typing.Union[
    typing.List[
        typing.Union[
            cluster_initial_member.ClusterInitialMemberDictInput,
            cluster_initial_member.ClusterInitialMemberDict,
        ],
    ],
    typing.Tuple[
        typing.Union[
            cluster_initial_member.ClusterInitialMemberDictInput,
            cluster_initial_member.ClusterInitialMemberDict,
        ],
        ...
    ]
]


@dataclasses.dataclass(frozen=True)
class Members(
    schemas.Schema[schemas.immutabledict, MembersTuple]
):
    types: typing.FrozenSet[typing.Type] = frozenset({tuple})
    items: typing.Type[cluster_initial_member.ClusterInitialMember] = dataclasses.field(default_factory=lambda: cluster_initial_member.ClusterInitialMember) # type: ignore
    type_to_output_cls: typing.Mapping[
        typing.Type,
        typing.Type
    ] = dataclasses.field(
        default_factory=lambda: {
            tuple: MembersTuple
        }
    )

    @classmethod
    def validate(
        cls,
        arg: typing.Union[
            MembersTupleInput,
            MembersTuple,
        ],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> MembersTuple:
        return super().validate_base(
            arg,
            configuration=configuration,
        )
Properties = typing.TypedDict(
    'Properties',
    {
        "members": typing.Type[Members],
        "joinerKit": typing.Type[JoinerKit],
    }
)
